/* Generated: Mon Jan 06 2025 14:59:55 GMT-0700 (Mountain Standard Time) */

static void CCONV PhidgetLEDArray_errorHandler(PhidgetChannelHandle ch, Phidget_ErrorEventCode code);
static void CCONV PhidgetLEDArray_free(PhidgetChannelHandle *ch);
static PhidgetReturnCode CCONV PhidgetLEDArray_bridgeInput(PhidgetChannelHandle phid, BridgePacket *bp);
static PhidgetReturnCode CCONV PhidgetLEDArray_setStatus(PhidgetChannelHandle phid, BridgePacket *bp);
static PhidgetReturnCode CCONV PhidgetLEDArray_getStatus(PhidgetChannelHandle phid, BridgePacket **bp);
static PhidgetReturnCode CCONV PhidgetLEDArray_initAfterOpen(PhidgetChannelHandle phid);
static PhidgetReturnCode CCONV PhidgetLEDArray_setDefaults(PhidgetChannelHandle phid);
static void CCONV PhidgetLEDArray_fireInitialEvents(PhidgetChannelHandle phid);
static int CCONV PhidgetLEDArray_hasInitialState(PhidgetChannelHandle phid);

struct _PhidgetLEDArray {
	struct _PhidgetChannel phid;
	int enabled;
	uint32_t minFadeTime;
	uint32_t maxFadeTime;
	uint32_t minLEDCount;
	uint32_t maxLEDCount;
	PhidgetLEDArray_Protocol protocol;
};

static PhidgetReturnCode CCONV
_setStatus(PhidgetChannelHandle phid, BridgePacket *bp) {
	PhidgetLEDArrayHandle ch;
	int nsversion;

	ch = (PhidgetLEDArrayHandle)phid;

	nsversion = getBridgePacketUInt32ByName(bp, "_class_version_");
	if (nsversion != 3) {
		loginfo("%"PRIphid": server/client class version mismatch: %d != 3 - functionality may be limited.", phid, nsversion);
	}

	if (hasBridgePacketByName(bp, "enabled"))
		ch->enabled = getBridgePacketInt32ByName(bp, "enabled");
	if (hasBridgePacketByName(bp, "minFadeTime"))
		ch->minFadeTime = getBridgePacketUInt32ByName(bp, "minFadeTime");
	if (hasBridgePacketByName(bp, "maxFadeTime"))
		ch->maxFadeTime = getBridgePacketUInt32ByName(bp, "maxFadeTime");
	if (hasBridgePacketByName(bp, "minLEDCount"))
		ch->minLEDCount = getBridgePacketUInt32ByName(bp, "minLEDCount");
	if (hasBridgePacketByName(bp, "maxLEDCount"))
		ch->maxLEDCount = getBridgePacketUInt32ByName(bp, "maxLEDCount");
	if (hasBridgePacketByName(bp, "protocol"))
		ch->protocol = getBridgePacketInt32ByName(bp, "protocol");

	return (EPHIDGET_OK);
}

static PhidgetReturnCode CCONV
_getStatus(PhidgetChannelHandle phid, BridgePacket **bp) {
	PhidgetLEDArrayHandle ch;

	ch = (PhidgetLEDArrayHandle)phid;

	return (createBridgePacket(bp, BP_SETSTATUS, 7, "_class_version_=%u"
	  ",enabled=%d"
	  ",minFadeTime=%u"
	  ",maxFadeTime=%u"
	  ",minLEDCount=%u"
	  ",maxLEDCount=%u"
	  ",protocol=%d"
	  ,3 /* class version */
	  ,ch->enabled
	  ,ch->minFadeTime
	  ,ch->maxFadeTime
	  ,ch->minLEDCount
	  ,ch->maxLEDCount
	  ,ch->protocol
	));
}

static PhidgetReturnCode CCONV
_bridgeInput(PhidgetChannelHandle phid, BridgePacket *bp) {
	PhidgetLEDArrayHandle ch;
	PhidgetReturnCode res;

	ch = (PhidgetLEDArrayHandle)phid;
	res = EPHIDGET_OK;

	switch (bp->vpkt) {
	case BP_SETANIMATION:
		res = DEVBRIDGEINPUT(phid, bp);
		break;
	case BP_DATAOUT:
		res = DEVBRIDGEINPUT(phid, bp);
		break;
	case BP_SYNCHRONIZE:
		res = DEVBRIDGEINPUT(phid, bp);
		break;
	case BP_SETENABLED:
		TESTBOOL_IOP(bp->iop, getBridgePacketInt32(bp, 0));
		res = DEVBRIDGEINPUT(phid, bp);
		if (res != EPHIDGET_OK)
			break;
		ch->enabled = getBridgePacketInt32(bp, 0);
		if (bridgePacketIsFromNet(bp)) {
			FIRE_PROPERTYCHANGE(ch, "Enabled");
		}
		break;
	case BP_SETPROTOCOL:
		if (!supportedLEDArrayProtocol(phid, (PhidgetLEDArray_Protocol)getBridgePacketInt32(bp, 0)))
			return (MOS_ERROR(bp->iop, EPHIDGET_INVALIDARG,
			  "Specified LEDArrayProtocol is unsupported by this device."));
		res = DEVBRIDGEINPUT(phid, bp);
		if (res != EPHIDGET_OK)
			break;
		ch->protocol = getBridgePacketInt32(bp, 0);
		if (bridgePacketIsFromNet(bp)) {
			FIRE_PROPERTYCHANGE(ch, "Protocol");
		}
		break;
	default:
		logerr("%"PRIphid": unsupported bridge packet:0x%x", phid, bp->vpkt);
		res = EPHIDGET_UNSUPPORTED;
	}

	return (res);
}

static PhidgetReturnCode CCONV
_initAfterOpen(PhidgetChannelHandle phid) {
	PhidgetReturnCode ret;

	TESTPTR(phid);

	ret = EPHIDGET_OK;


	switch (phid->UCD->uid) {
	default:
		MOS_PANIC("Unsupported Channel");
	}


	return (ret);
}

static PhidgetReturnCode CCONV
_setDefaults(PhidgetChannelHandle phid) {
	PhidgetReturnCode ret;

	TESTPTR(phid);

	ret = EPHIDGET_OK;

	switch (phid->UCD->uid) {
	default:
		MOS_PANIC("Unsupported Channel");
	}

	return (ret);
}

static void CCONV
_fireInitialEvents(PhidgetChannelHandle phid) {

}

static int CCONV
_hasInitialState(PhidgetChannelHandle phid) {

	return (PTRUE);
}

static void CCONV
_free(PhidgetChannelHandle *ch) {

	mos_free(*ch, sizeof (struct _PhidgetLEDArray));
}

static PhidgetReturnCode CCONV
_create(PhidgetLEDArrayHandle *phidp) {

	CHANNELCREATE_BODY(LEDArray, PHIDCHCLASS_LEDARRAY);
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetLEDArray_delete(PhidgetLEDArrayHandle *phidp) {

	return (Phidget_delete((PhidgetHandle *)phidp));
}

API_PRETURN
PhidgetLEDArray_synchronizeAnimation(PhidgetLEDArrayHandle ch) {

	TESTPTR_PR(ch);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	return bridgeSendToDevice((PhidgetChannelHandle)ch, BP_SYNCHRONIZE, NULL, NULL, 0, NULL); 
}

API_PRETURN
PhidgetLEDArray_setEnabled(PhidgetLEDArrayHandle ch, int enabled) {

	TESTPTR_PR(ch);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	return (bridgeSendToDevice((PhidgetChannelHandle)ch, BP_SETENABLED, NULL, NULL, 1, "%d", enabled));
}

API_PRETURN
PhidgetLEDArray_getEnabled(PhidgetLEDArrayHandle ch, int *enabled) {

	TESTPTR_PR(ch);
	TESTPTR_PR(enabled);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	*enabled = ch->enabled;
	if (ch->enabled == (int)PUNK_BOOL)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetLEDArray_getMinFadeTime(PhidgetLEDArrayHandle ch, uint32_t *minFadeTime) {

	TESTPTR_PR(ch);
	TESTPTR_PR(minFadeTime);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	*minFadeTime = ch->minFadeTime;
	if (ch->minFadeTime == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetLEDArray_getMaxFadeTime(PhidgetLEDArrayHandle ch, uint32_t *maxFadeTime) {

	TESTPTR_PR(ch);
	TESTPTR_PR(maxFadeTime);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	*maxFadeTime = ch->maxFadeTime;
	if (ch->maxFadeTime == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetLEDArray_getMinLEDCount(PhidgetLEDArrayHandle ch, uint32_t *minLEDCount) {

	TESTPTR_PR(ch);
	TESTPTR_PR(minLEDCount);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	*minLEDCount = ch->minLEDCount;
	if (ch->minLEDCount == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetLEDArray_getMaxLEDCount(PhidgetLEDArrayHandle ch, uint32_t *maxLEDCount) {

	TESTPTR_PR(ch);
	TESTPTR_PR(maxLEDCount);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	*maxLEDCount = ch->maxLEDCount;
	if (ch->maxLEDCount == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetLEDArray_setProtocol(PhidgetLEDArrayHandle ch, PhidgetLEDArray_Protocol protocol) {

	TESTPTR_PR(ch);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	return (bridgeSendToDevice((PhidgetChannelHandle)ch, BP_SETPROTOCOL, NULL, NULL, 1, "%d",
	  protocol));
}

API_PRETURN
PhidgetLEDArray_getProtocol(PhidgetLEDArrayHandle ch, PhidgetLEDArray_Protocol *protocol) {

	TESTPTR_PR(ch);
	TESTPTR_PR(protocol);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	*protocol = ch->protocol;
	if (ch->protocol == (PhidgetLEDArray_Protocol)PUNK_ENUM)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}
